package resolvers

import (
	"context"
	"fmt"
	"strconv"

	"github.com/graphql-go/graphql"
	"github.com/ttacon/libphonenumber"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers/errs"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

type FavoriteResponse struct {
	Account   *account.Account
	Favorites []*property.Property
}

type UnfavoriteResponse struct {
	Account   *account.Account
	Favorites []*property.Property
}

type ValidatePhoneNumberInput struct {
	Phone  string `json:"phone"`
	Region string `json:"region"`
}

type ValidatePhoneNumber struct {
	CountryCode    int32  `json:"country_code"`
	NationalNumber string `json:"national_number"`
	E164           string `json:"e164"`
	Error          string `json:"error"`
}

func initAccount(rootQueries, rootMutations *graphql.Fields) {
	validation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "AccountValidation",
		Description: "Account validation root",
		Fields: graphql.Fields{
			"phone": &graphql.Field{
				Description: "validate phone number",
				Args:        util.BindArguments(ValidatePhoneNumberInput{}),
				Resolve:     AccountQueryValidationPhoneResolver,
				Type:        util.BindOutput(ValidatePhoneNumber{}, "phone validation response", nil),
			},
		},
	}))

	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "AccountQuery",
		Description: "Account query root",
		Fields: graphql.Fields{
			"info": &graphql.Field{
				Description: "Returns current user profile info",
				Resolve:     AccountQueryInfoResolver,
				Type:        util.BindOutput(account.Account{}, "Authenticated user profile information", nil),
			},
			"whois": &graphql.Field{
				Description: "Returns other user limited profile info",
				Args:        util.BindArguments(account.GetRequest{}),
				Resolve:     AccountQueryWhoisResolver,
				Type:        util.BindOutput(account.Account{}, "Authenticated user profile information", nil),
			},
			"get": &graphql.Field{
				Description: "Get account by id",
				Args:        util.BindArguments(account.GetRequest{}),
				Resolve:     AccountQueryGetResolver,
				Type:        util.BindOutput(account.GetResponse{}, "Account details", nil),
			},
			"list": &graphql.Field{
				Description: "Get account list",
				Resolve:     AccountQueryListResolver,
				Type:        util.BindOutput(account.ServiceListResponse{}, "Account list", nil),
			},
			"listWistiaVideos": &graphql.Field{
				Description: "Get wistia videos list",
				Resolve:     AccountQueryListWistiaVideosResolver,
				Type:        util.BindOutput(account.ListWistiaVideosResponse{}, "Wistia videos list", nil),
			},
			"validation": &graphql.Field{
				Type: validation,
				Resolve: func(p graphql.ResolveParams) (interface{}, error) {
					return validation, nil
				},
			},
		},
	}))

	(*rootQueries)["account"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "AccountMutation",
		Description: "Account mutation root",
		Fields: graphql.Fields{
			"favorite": &graphql.Field{
				Description: "Favorites an object",
				Resolve:     AccountMutationFavoriteResolver,
				Args:        util.BindArguments(account.FavoriteRequest{}),
				Type:        util.BindOutput(FavoriteResponse{}, "Authenticated user profile information", nil),
			},
			"unfavorite": &graphql.Field{
				Description: "Unfavorites an object",
				Args:        util.BindArguments(account.UnfavoriteRequest{}),
				Resolve:     AccountMutationUnfavoriteResolver,
				Type:        util.BindOutput(UnfavoriteResponse{}, "Authenticated user profile information", nil),
			},
			"delete": &graphql.Field{
				Description: "Delete account",
				Args:        util.BindArguments(account.DeleteAccountRequest{}),
				Resolve:     AccountMutationDeleteResolver,
				Type:        util.BindOutput(account.DeleteAccountResponse{}, "Delete response", nil),
			},
			"restore": &graphql.Field{
				Description: "Restore account",
				Args:        util.BindArguments(account.RestoreAccountRequest{}),
				Resolve:     AccountMutationRestoreResolver,
				Type:        util.BindOutput(account.RestoreAccountResponse{}, "Restore response", nil),
			},
			"changebankdetails": &graphql.Field{
				Description: "Change bank details of the account",
				Args:        util.BindArguments(account.ChangeBankDetailsRequest{}),
				Resolve:     AccountMutationChangeBankDetailsResolver,
				Type:        util.BindOutput(account.ChangeBankDetailsResponse{}, "Change bank details response", nil),
			},
			"wistiaVideoViewed": &graphql.Field{
				Description: "Store viewed video in account model",
				Args:        util.BindArguments(account.WistiaVideoViewedRequest{}),
				Resolve:     AccountMutationWistiaVideoViewedResolver,
				Type:        util.BindOutput(account.WistiaVideoViewedResponse{}, "Wistia video viewed response", nil),
			},
		},
	}))

	(*rootMutations)["account"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func AccountMutationChangeBankDetailsResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &account.ChangeBankDetailsRequest{}

	err := util.UnmarshalArguments(params.Args, request)
	if err != nil {
		return nil, err
	}

	if remote, ok := params.Context.Value(ws.CtxRemote).(string); ok {
		request.Remote = remote
	}
	request.User = user.GetAccount().GetIdValue()

	return connections.AccountService.ChangeBankDetails(params.Context, request)
}

func AccountQueryInfoResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	resp, err := connections.AccountService.Get(params.Context, &account.GetRequest{
		Id:    user.GetAccount().GetIdValue(),
		Admin: false,
	})
	return resp.GetAccount(), err
}

func AccountQueryValidationPhoneResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &ValidatePhoneNumberInput{}

	err := util.UnmarshalArguments(params.Args, request)
	if err != nil {
		return nil, err
	}

	var resp *ValidatePhoneNumber

	parsed, err := libphonenumber.Parse(request.Phone, request.Region)
	if err != nil {
		resp = &ValidatePhoneNumber{
			Error: err.Error(),
		}
	} else {
		resp = &ValidatePhoneNumber{
			CountryCode:    parsed.GetCountryCode(),
			NationalNumber: strconv.FormatUint(parsed.GetNationalNumber(), 10),
			E164:           fmt.Sprintf("+%d%d", parsed.GetCountryCode(), parsed.GetNationalNumber()),
		}
	}

	return resp, nil
}

func AccountQueryWhoisResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &account.GetRequest{
		Admin: false,
	}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.AccountService.Get(params.Context, request)
	if err != nil {
		return nil, err
	}
	return &account.Account{
		Id:        resp.GetAccount().GetId(),
		CreatedAt: resp.GetAccount().GetCreatedAt(),
		Details:   resp.GetAccount().GetDetails(),
	}, nil
}

func AccountQueryListWistiaVideosResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &account.ListWistiaVideosRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AccountService.ListWistiaVideos(params.Context, request)
}

func AccountMutationFavoriteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &account.FavoriteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.AccountService.Favorite(params.Context, request)
	if err != nil {
		return nil, err
	}
	props := getSavedProperties(params.Context, resp.GetAccount())

	req := &FavoriteResponse{
		Account:   resp.GetAccount(),
		Favorites: props,
	}

	return req, nil
}

func AccountMutationUnfavoriteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &account.UnfavoriteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.AccountService.Unfavorite(params.Context, request)
	if err != nil {
		return nil, err
	}
	props := getSavedProperties(params.Context, resp.GetAccount())

	req := &UnfavoriteResponse{
		Account:   resp.GetAccount(),
		Favorites: props,
	}

	return req, nil
}

func AccountQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, AdminRole); err != nil {
		return nil, err
	}
	request := &account.GetRequest{
		Admin: false,
	}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AccountService.Get(params.Context, request)
}

func AccountQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := CheckPermissions(params.Context, AdminRole); err != nil {
		return nil, err
	}
	request := &account.ServiceListRequest{
		Admin: false,
	}
	return connections.AccountService.ServiceList(params.Context, request)
}

func AccountMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &account.DeleteAccountRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.AccountService.DeleteAccount(params.Context, request)
}

func AccountMutationRestoreResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &account.RestoreAccountRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.AccountService.RestoreAccount(params.Context, request)
}

func AccountMutationWistiaVideoViewedResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &account.WistiaVideoViewedRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.UserId = user.GetAccount().GetIdValue()
	return connections.AccountService.WistiaVideoViewed(params.Context, request)
}

func getSavedProperties(ctx context.Context, acc *account.Account) []*property.Property {
	var props []*property.Property
	if acc == nil {
		return props
	}

	if acc.GetFavoritesValue() == nil {
		return props
	}

	if acc.GetFavoritesValue().GetProperties() == nil {
		return props
	}

	if len(acc.GetFavoritesValue().GetProperties().GetValue()) == 0 {
		return props
	}

	for _, p := range acc.GetFavoritesValue().GetProperties().GetValue() {
		resp, err := connections.PropertyService.Get(ctx, &property.GetRequest{
			Id: p,
		})
		if err != nil {
			continue
		}
		props = append(props, resp.GetProperty())
	}
	return props
}
