package admin

import (
	"github.com/graphql-go/graphql"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	q "gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers"
)

type blockUserRequest struct {
	ID string `json:"id,omitempty"`
}

type allAdminsResult struct {
	Data  []*account.Account `json:"data,omitempty"`
	Total int32              `json:"total,omitempty"`
}

func initAdmin(query, mutation *graphql.Fields) {
	(*query)["Admin"] = &graphql.Field{
		Description: "Get admin by id",
		Args:        util.BindArguments(getParams{}),
		Resolve:     adminQueryGetAdminResolver,
		Type:        util.BindOutput(account.Account{}, "Admin details", nil),
	}
	(*query)["allAdmins"] = &graphql.Field{
		Description: "Returns admin list",
		Args:        util.BindArguments(allParams{}),
		Resolve:     adminQueryAllAdminsResolver,
		Type:        util.BindOutput(allAdminsResult{}, "Admin list", nil),
	}

	(*mutation)["createAdmin"] = &graphql.Field{
		Description: "Create admin account",
		Args:        util.BindArguments(auth.SignupRequest{}),
		Resolve:     adminMutationCreateAdminResolver,
		Type:        util.BindOutput(account.Account{}, "Admin create response", nil),
	}
	(*mutation)["deleteAdmin"] = &graphql.Field{
		Description: "Delete admin account",
		Args:        util.BindArguments(account.ServiceDeleteRequest{}),
		Resolve:     adminMutationDeleteAdminResolver,
		Type:        util.BindOutput(account.ServiceDeleteResponse{}, "Account delete response", nil),
	}
	(*mutation)["updateAdmin"] = &graphql.Field{
		Description: "Update admin account",
		Args:        util.BindArguments(account.ServiceUpdateRequest{}),
		Resolve:     adminMutationUpdateAdminResolver,
		Type:        util.BindOutput(account.Account{}, "Account update response", nil),
	}
	(*mutation)["blockAdmin"] = &graphql.Field{
		Description: "Block admin account",
		Args:        util.BindArguments(blockUserRequest{}),
		Resolve:     adminMutationBlockAdminResolver,
		Type:        util.BindOutput(auth.BlockUserResponse{}, "Block Admin response", nil),
	}
	(*mutation)["unblockAdmin"] = &graphql.Field{
		Description: "Unblock admin account",
		Args:        util.BindArguments(blockUserRequest{}),
		Resolve:     adminMutationUnblockAdminResolver,
		Type:        util.BindOutput(auth.BlockUserResponse{}, "Unblock Admin response", nil),
	}
	(*mutation)["addConfirmationAdmin"] = &graphql.Field{
		Description: "Add conformation information",
		Args:        util.BindArguments(account.ConfirmationRequest{}),
		Resolve:     adminMutationConfirmation,
		Type:        util.BindOutput(account.ConfirmationResponse{}, "Add conformation information", nil),
	}
	(*mutation)["resetCaptcha"] = &graphql.Field{
		Description: "Reset login attempts",
		Args:        util.BindArguments(auth.ResetAllAuthAttemptsRequest{}),
		Resolve:     adminMutationResetCaptcha,
		Type:        util.BindOutput(auth.ResetAllAuthAttemptsResponse{}, "Reset login attempts result", nil),
	}
}

func adminMutationCreateAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &auth.SignupRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.AdminSite = &auth.SignupRequest_AdminSiteValue{AdminSiteValue: true}
	request.Account.Admin = &account.Account_AdminValue{AdminValue: true}
	remote := params.Context.Value(ws.CtxRemote).(string)
	request.Remote = &auth.SignupRequest_RemoteValue{RemoteValue: remote}
	resp, err := connections.AuthService.Create(params.Context, request)
	if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
		return nil, err
	}
	return resp.GetAccount(), nil
}

func adminQueryGetAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &account.GetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Admin = true
	resp, err := connections.AccountService.Get(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp.GetAccount(), nil
}

func adminQueryAllAdminsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	input := &allParams{}
	if err := util.UnmarshalArguments(params.Args, input); err != nil {
		return nil, err
	}
	req := &q.QueryParams{
		Filter: input.Filter,
		Sort:   input.Sort,
		Pagination: &q.QueryParamsPagination{
			Page:    input.Pagination.Page,
			PerPage: input.Pagination.PerPage,
		},
	}
	request := &account.AccountListFilterRequest{
		Params: &account.AccountListFilterRequest_ParamsValue{ParamsValue: req},
	}
	request.Admin = true
	resp, err := connections.AccountService.ListFilter(params.Context, request)
	if err != nil {
		return nil, err
	}
	accounts := resp.GetAccounts()
	total := resp.GetTotal()
	return allUsersResult{
		Data:  accounts,
		Total: total,
	}, nil
}

func adminMutationUpdateAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &account.ServiceUpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	reqBlock := &auth.BlockUserRequest{
		Id:    request.Account.GetIdValue(),
		Block: request.Account.GetBlockValue(),
	}
	_, err := connections.AuthService.BlockUser(params.Context, reqBlock)
	if err != nil {
		return nil, err
	}

	resp, err := connections.AccountService.ServiceUpdate(params.Context, request)
	if err != nil {
		return nil, err
	}

	if request.GetNewPassword() != nil {
		resp, err := connections.AuthService.ServiceUpdatePassword(params.Context, &auth.ServiceUpdatePasswordRequest{
			UserId:      request.GetAccount().GetIdValue(),
			NewPassword: request.GetNewPasswordValue(),
		})

		if err != nil || resp == nil || !resp.Success || len(resp.Errors) > 0 {
			return resp, err
		}
	}

	return resp.GetAccount(), nil
}

func adminMutationConfirmation(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &account.ConfirmationRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.AccountService.AddConfirmation(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func adminMutationDeleteAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &account.ServiceDeleteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.AccountService.ServiceDelete(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func adminMutationBlockAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &auth.BlockUserRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Block = true

	resp, err := connections.AuthService.BlockUser(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func adminMutationUnblockAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &auth.BlockUserRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Block = false

	resp, err := connections.AuthService.BlockUser(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func adminMutationResetCaptcha(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &auth.ResetAllAuthAttemptsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.AuthService.ResetAllAuthAttempts(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
