package admin

import (
	"context"
	"os"

	"github.com/graphql-go/graphql"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

func initDocument(query, mutation *graphql.Fields) {
	(*query)["Document"] = &graphql.Field{
		Description: "Get document by id",
		Args:        util.BindArguments(document.GetRequest{}),
		Resolve:     adminQueryGetDocumentResolver,
		Type:        util.BindOutput(document.GetResponse{}, "Document with given id", nil),
	}
	(*query)["allDocuments"] = &graphql.Field{
		Description: "Get documents by filters",
		Args:        util.BindArguments(document.ListRequest{}),
		Resolve:     adminQueryAllDocumentsResolver,
		Type:        util.BindOutput(document.ListResponse{}, "Document list", nil),
	}

	(*mutation)["createDocument"] = &graphql.Field{
		Description: "Create document",
		Args:        util.BindArguments(document.ServiceCreateRequest{}),
		Resolve:     adminMutationCreateDocumentResolver,
		Type:        util.BindOutput(document.CreateResponse{}, "Document create response", nil),
	}
	(*mutation)["updateDocument"] = &graphql.Field{
		Description: "Update document",
		Args:        util.BindArguments(document.ServiceUpdateRequest{}),
		Resolve:     adminMutationUpdateDocumentResolver,
		Type:        util.BindOutput(document.ServiceUpdateResponse{}, "Document update response", nil),
	}
	(*mutation)["updateDocumentState"] = &graphql.Field{
		Description: "Update document state",
		Args:        util.BindArguments(document.UpdateStatusRequest{}),
		Resolve:     adminMutationUpdateDocumentStateResolver,
		Type:        util.BindOutput(document.UpdateStatusResponse{}, "Document state update response", nil),
	}
	(*mutation)["deleteDocument"] = &graphql.Field{
		Description: "Delete document",
		Args:        util.BindArguments(document.DeleteRequest{}),
		Resolve:     adminMutationDeleteDocumentResolver,
		Type:        util.BindOutput(document.DeleteResponse{}, "Document permanent delete", nil),
	}
	(*mutation)["cleanFilesCache"] = &graphql.Field{
		Description: "Clean document files cache",
		Args:        util.BindArguments(document.CleanFilesRequest{}),
		Resolve:     adminMutationCleanFilesCacheResolver,
		Type:        graphql.Boolean,
	}
	(*mutation)["cleanThumbsCache"] = &graphql.Field{
		DeprecationReason: "Use cleanFilesCache",
		Description:       "Clean thumbs cache",
		Args:              util.BindArguments(document.CleanThumbsCacheRequest{}),
		Resolve:           adminMutationCleanThumbsCacheResolver,
		Type:              util.BindOutput(document.CleanThumbsCacheResponse{}, "Clean thumbs cache response", nil),
	}
	(*mutation)["cleanWebpCache"] = &graphql.Field{
		DeprecationReason: "Use cleanFilesCache",
		Description:       "Clean webp cache",
		Args:              util.BindArguments(document.CleanThumbsCacheRequest{}),
		Resolve:           adminMutationCleanWebpCacheResolver,
		Type:              util.BindOutput(document.CleanWebpCacheResponse{}, "Clean webp cache response", nil),
	}
	(*mutation)["fileSoftDeleteByDocument"] = &graphql.Field{
		DeprecationReason: "Use updateDocument with removed=true or deleteDocument",
		Description:       "File soft delete by document id",
		Args:              util.BindArguments(document.FileSoftDeleteByDocumentRequest{}),
		Resolve:           adminMutationFileSoftDeleteByDocumentResolver,
		Type: util.BindOutput(document.FileSoftDeleteByDocumentResponse{},
			"file soft delet by document response", nil),
	}
}

func adminQueryGetDocumentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &document.GetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	doc, err := connections.DocumentService.Get(params.Context, request)
	if err != nil {
		return nil, err
	}
	return doc.GetDocument(), err
}

func adminQueryAllDocumentsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &document.ListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	if request.GetProperty() != "" {
		resp, err := connections.PropertyService.Get(params.Context, &property.GetRequest{
			Id: request.GetProperty(),
		})
		if err == nil {
			request.User = resp.GetProperty().GetPeople().GetLandlord().GetValue()
		}
	}
	docs, err := connections.DocumentService.List(params.Context, request)
	if err != nil {
		return nil, err
	}

	return docs, nil
}

func adminMutationCreateDocumentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &document.ServiceCreateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	if request.User == "" {
		reqProp := &property.GetRequest{
			Id: request.Property,
		}
		prop, err := connections.PropertyService.GetAdminVerify(params.Context, reqProp)
		if err != nil {
			return nil, err
		}
		people := prop.GetProperty().GetPeople()
		if people.GetLandlord() != nil {
			request.User = people.GetLandlord().GetValue()
		} else if people.GetManager() != nil {
			request.User = people.GetManager().GetValue()
		}
	}

	return connections.DocumentService.ServiceCreate(params.Context, request)
}

func adminMutationUpdateDocumentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &document.ServiceUpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	if request.User == "" {
		reqProp := &property.GetRequest{
			Id: request.Property,
		}
		prop, err := connections.PropertyService.GetAdminVerify(params.Context, reqProp)
		if err != nil {
			return nil, err
		}
		people := prop.GetProperty().GetPeople()
		if people.GetLandlord() != nil {
			request.User = people.GetLandlord().GetValue()
		} else if people.GetManager() != nil {
			request.User = people.GetManager().GetValue()
		}
	}

	resp, err := connections.DocumentService.ServiceUpdate(params.Context, request)
	if err != nil {
		return nil, err
	}

	err = adminPropertyUpdateMedia(
		params.Context,
		request.GetProperty(),
		request.GetFile(),
		request.GetType(),
		request.GetRemoved(),
	)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationDeleteDocumentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &document.DeleteRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.DocumentService.Delete(params.Context, request)
}

func adminMutationUpdateDocumentStateResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &document.UpdateStatusRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.DocumentService.UpdateStatus(params.Context, request)
}

func adminMutationCleanFilesCacheResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &document.CleanFilesRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	_, err := connections.DocumentService.CleanFiles(params.Context, &document.CleanFilesRequest{
		Size: request.GetSize(),
	})
	if err != nil {
		return nil, err
	}

	switch request.GetSize() {
	case document.ImageSize_THUMBNAIL:
		err = os.RemoveAll(MediaCacheDir + "/thumb")
		if err != nil {
			return nil, err
		}

		return true, nil
	case document.ImageSize_WEBP:
		err = os.RemoveAll(MediaCacheDir + "/webp")
		if err != nil {
			return nil, err
		}

		return true, nil

	default:
		return false, nil
	}
}

func adminMutationCleanThumbsCacheResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	resp, err := connections.DocumentService.CleanFiles(params.Context, &document.CleanFilesRequest{
		Size: document.ImageSize_THUMBNAIL,
	})
	if err != nil {
		return nil, err
	}

	err = os.RemoveAll(MediaCacheDir + "/thumb")
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationCleanWebpCacheResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	resp, err := connections.DocumentService.CleanFiles(params.Context, &document.CleanFilesRequest{
		Size: document.ImageSize_WEBP,
	})
	if err != nil {
		return nil, err
	}

	err = os.RemoveAll(MediaCacheDir + "/webp")
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminPropertyUpdateMedia(ctx context.Context, prop, file string, typ document.DocumentType, rem bool) (err error) {
	if prop == "" {
		return
	}

	var photos *property.FileChanges

	switch typ {
	case document.DocumentType_PHOTO, document.DocumentType_PHOTO_RAW:
		if rem {
			photos = &property.FileChanges{
				Removed: []string{file},
			}
		} else {
			photos = &property.FileChanges{
				Added: []string{file},
			}
		}
	}

	if photos != nil {
		_, err = connections.PropertyService.UpdatePublications(ctx, &property.UpdatePublicationsRequest{
			Property: prop,
			Photos:   photos,
			Plans:    photos,
		})
	}

	return
}

func adminMutationFileSoftDeleteByDocumentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &document.FileSoftDeleteByDocumentRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	doc, err := connections.DocumentService.Get(params.Context, &document.GetRequest{
		Id: request.GetDocument(),
	})
	if err != nil {
		return nil, err
	}

	resp, err := connections.DocumentService.ServiceUpdate(params.Context, &document.ServiceUpdateRequest{
		Id:          doc.GetDocument().GetId(),
		Type:        doc.GetDocument().GetType(),
		User:        doc.GetDocument().GetUser(),
		Property:    doc.GetDocument().GetProperty(),
		File:        doc.GetDocument().GetFile().GetId(),
		Expiration:  doc.GetDocument().GetExpiration(),
		Description: doc.GetDocument().GetDescription(),
		State:       doc.GetDocument().GetState(),
		Code:        doc.GetDocument().GetCode(),
		Amount:      doc.GetDocument().GetAmount(),
		Removed:     true,
	})
	if err != nil {
		return nil, err
	}

	err = adminPropertyUpdateMedia(
		params.Context,
		doc.GetDocument().GetProperty(),
		doc.GetDocument().GetFile().GetId(),
		doc.GetDocument().GetType(),
		true,
	)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
