package admin

import (
	"github.com/graphql-go/graphql"
	q "gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
)

type allInvitationsResult struct {
	Data  []*invitation.InvitationInfo `json:"data,omitempty"`
	Total int32                        `json:"total,omitempty"`
}

func initInvitation(query, mutation *graphql.Fields) {
	(*query)["Invitation"] = &graphql.Field{
		Description: "Retrieves specified invitation",
		Args:        util.BindArguments(invitation.GetInvitationRequest{}),
		Resolve:     adminQueryGetInvitationResolver,
		Type:        util.BindOutput(invitation.InvitationInfo{}, "Invitation information", nil),
	}
	(*query)["allInvitations"] = &graphql.Field{
		Description: "Retrieves invitation list",
		Args:        util.BindArguments(allParams{}),
		Resolve:     adminQueryAllInvitationsResolver,
		Type:        util.BindOutput(allInvitationsResult{}, "Invitation list", nil),
	}
}

func adminQueryGetInvitationResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invitation.GetInvitationRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	inv, err := connections.InvitationService.GetInvitation(params.Context, request)
	if err != nil {
		return nil, err
	}

	return inv.Info, nil
}

func adminQueryAllInvitationsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	input := &allParams{}
	if err := util.UnmarshalArguments(params.Args, input); err != nil {
		return nil, err
	}

	req := &q.QueryParams{
		Filter: input.Filter,
		Sort:   input.Sort,
		Pagination: &q.QueryParamsPagination{
			Page:    input.Pagination.Page,
			PerPage: input.Pagination.PerPage,
		},
	}

	if input.Sort == nil {
		req.Sort = []*q.QueryParamsSort{{Field: "created_at", Order: -1}}
	}

	request := &invitation.InvitationListFilterRequest{
		Params: req,
	}

	request.AdminSite = true

	resp, err := connections.InvitationService.ListFilter(params.Context, request)
	if err != nil {
		return nil, err
	}

	invitations := resp.GetInvitations()

	total := resp.GetTotal()

	return &allInvitationsResult{
		Data:  invitations,
		Total: total,
	}, nil
}
