package admin

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers"
	stats "gitlab.mashroom.com/mashroom/backend/stats/api"
)

func initStats(query, mutation *graphql.Fields) {
	(*query)["getUploadJob"] = &graphql.Field{
		Description: "Retrieves existing upload job",
		Args:        util.BindArguments(stats.GetUploadRequest{}),
		Resolve:     adminQueryGetUploadJobResolver,
		Type:        util.BindOutput(stats.GetUploadResponse{}, "Upload jobs params", nil),
	}

	(*mutation)["createUploadJob"] = &graphql.Field{
		Description: "Create upload job",
		Args:        util.BindArguments(stats.CreateUploadRequest{}),
		Resolve:     adminMutationCreateUploadJobResolver,
		Type:        util.BindOutput(stats.CreateUploadResponse{}, "Create job status", nil),
	}
	(*mutation)["deleteUploadJob"] = &graphql.Field{
		Description: "Stop and delete existing upload job",
		Args:        util.BindArguments(stats.DeleteUploadRequest{}),
		Resolve:     adminMutationDeleteUploadJobResolver,
		Type:        util.BindOutput(stats.DeleteUploadResponse{}, "Delete job status", nil),
	}
	(*mutation)["accountUpload"] = &graphql.Field{
		Description: "Run upload accounts",
		Args:        util.BindArguments(stats.UploadRequest{}),
		Resolve:     adminMutationAccountUploadResolver,
		Type:        util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
	(*mutation)["offerUpload"] = &graphql.Field{
		Description: "Run upload offers",
		Args:        util.BindArguments(stats.UploadRequest{}),
		Resolve:     adminMutationOfferUploadResolver,
		Type:        util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
	(*mutation)["propertyUpload"] = &graphql.Field{
		Description: "Run upload properties",
		Args:        util.BindArguments(stats.UploadRequest{}),
		Resolve:     adminMutationPropertyUploadResolver,
		Type:        util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
}

func adminQueryGetUploadJobResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.GetUploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.GetUpload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationCreateUploadJobResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.CreateUploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.CreateUpload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationDeleteUploadJobResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.DeleteUploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.DeleteUpload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationAccountUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.UploadAccounts(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationOfferUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.UploadOffers(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationPropertyUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.UploadProperties(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
