package resolvers

import (
	"github.com/graphql-go/graphql"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	cardpayment "gitlab.mashroom.com/mashroom/backend/cardpayment/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	extras "gitlab.mashroom.com/mashroom/backend/extras/api"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers/errs"
	invoice "gitlab.mashroom.com/mashroom/backend/invoice/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

type MarkReceiptPaidRequest struct {
	ID string `json:"id,omitempty"`
}

type CardpaymentFormArguments struct {
	cardpayment.FormReceiptRequest
	Items []string `json:"items"`
}

func initCardPayment(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "CardPaymentQuery",
		Description: "Card payment query root",
		Fields: graphql.Fields{
			"user": &graphql.Field{
				DeprecationReason: "use invoice list",
				Description:       "Get current user receipts",
				Resolve:           CardPaymentQueryUserResolver,
				Type:              util.BindOutput(cardpayment.GetUserResponse{}, "Receipt list", nil),
			},
			"property": &graphql.Field{
				DeprecationReason: "Use property { products } enrichment or invoice list",
				Description:       "Get services list",
				Args:              util.BindArguments(cardpayment.GetPropertyRequest{}),
				Resolve:           CardPaymentQueryPropertyResolver,
				Type:              util.BindOutput(cardpayment.GetPropertyResponse{}, "Receipt list", nil),
			},
		},
	}))

	(*rootQueries)["cardpayment"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "CardPaymentMutation",
		Description: "Card payment mutation root",
		Fields: graphql.Fields{
			"form": &graphql.Field{
				DeprecationReason: "Do not use directly",
				Description:       "Form a receipt",
				Args:              util.BindArguments(CardpaymentFormArguments{}),
				Resolve:           CardPaymentMutationFormResolver,
				Type: util.BindOutput(
					cardpayment.FormReceiptResponse{},
					"Payment intent id and secret",
					nil,
				),
			},
			"cancel": &graphql.Field{
				DeprecationReason: "Do not use directly",
				Description:       "Cancel a receipt",
				Args:              util.BindArguments(cardpayment.CancelReceiptRequest{}),
				Resolve:           CardPaymentMutationCancelResolver,
				Type:              util.BindOutput(cardpayment.CancelReceiptResponse{}, "Cancellation status", nil),
			},
			"check": &graphql.Field{
				Description: "Check receipt status",
				Args:        util.BindArguments(cardpayment.CheckReceiptRequest{}),
				Resolve:     CardPaymentMutationCheckResolver,
				Type:        util.BindOutput(cardpayment.CheckReceiptResponse{}, "Actual state of receipt", nil),
			},
			"mark": &graphql.Field{
				DeprecationReason: "Do not use directly outside of testing environment",
				Description:       "Mark as paid",
				Args:              util.BindArguments(cardpayment.MarkReceiptPaidRequest{}),
				Resolve:           CardPaymentMutationMarkResolver,
				Type:              util.BindOutput(cardpayment.MarkReceiptPaidResponse{}, "", nil),
			},
			"markbank": &graphql.Field{
				DeprecationReason: "Do not use directly outside of testing environment",
				Description:       "Mark as paid via bank",
				Args:              util.BindArguments(cardpayment.MarkReceiptPaidRequest{}),
				Resolve:           CardPaymentMutationMarkBankResolver,
				Type:              util.BindOutput(cardpayment.MarkReceiptPaidResponse{}, "", nil),
			},
		},
	}))

	(*rootMutations)["cardpayment"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func CardPaymentQueryUserResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &cardpayment.GetUserRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	var products []*invoice.Product

	resp, err := connections.InvoiceService.InvoiceList(params.Context, &invoice.InvoiceListRequest{
		User: user.GetAccount().GetIdValue(),
		Statuses: []invoice.InvoiceStatus{
			invoice.InvoiceStatus_INVOICE_FULFILLED,
		},
	})
	if err != nil {
		return products, nil
	}

	return &cardpayment.GetUserResponse{
		Payments: RenderCardpaymentReceipts(resp.GetInvoices()),
	}, nil
}

func RenderCardpaymentReceipts(invoices []*invoice.Invoice) (receipts []*cardpayment.Receipt) {
	for _, inv := range invoices {
		for _, p := range inv.GetPayments() {
			if p.GetSystem().GetStripe() == nil ||
				(p.GetStatus() != invoice.PaymentStatus_PAYMENT_SUCCESS &&
					p.GetStatus() != invoice.PaymentStatus_PAYMENT_CONFIRM) {
				continue
			}

			var services []*extras.ServiceSnapshot

			for _, s := range inv.GetServices() {
				if s.GetService().GetServiceId() == "" {
					continue
				}

				services = append(services, s.GetService())
			}

			if len(services) == 0 {
				continue
			}

			var status cardpayment.PaymentStatus

			switch p.GetStatus() {
			case invoice.PaymentStatus_PAYMENT_SUCCESS:
				status = cardpayment.PaymentStatus_SUCCEEDED
			case invoice.PaymentStatus_PAYMENT_PENDING:
				status = cardpayment.PaymentStatus_UNPAID
			case invoice.PaymentStatus_PAYMENT_FAILED:
				status = cardpayment.PaymentStatus_FAILED
			case invoice.PaymentStatus_PAYMENT_CONFIRM:
				status = cardpayment.PaymentStatus_PROCESSING
			case invoice.PaymentStatus_PAYMENT_CANCELED:
				status = cardpayment.PaymentStatus_CANCELLED // nolint:misspell // `CANCELLED` is external misspelling
			}

			receipts = append(receipts, &cardpayment.Receipt{
				Id:            p.GetPaymentId(),
				Type:          p.GetSystem().GetStripe().GetRequest().GetType(),
				ClientSecret:  p.GetSystem().GetStripe().GetResponse().GetClientSecret(),
				UserId:        inv.GetUser(),
				UserEmail:     "",
				Items:         services,
				Property:      inv.GetProperty(),
				Status:        status,
				CreatedAt:     p.GetCreatedAt(),
				UpdatedAt:     p.GetUpdatedAt(),
				InvoiceNumber: inv.GetInvoiceId(),
			})
		}
	}

	return
}

func CardPaymentQueryPropertyResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &cardpayment.GetPropertyRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	var products []*invoice.Product

	resp, err := connections.InvoiceService.InvoiceList(params.Context, &invoice.InvoiceListRequest{
		User:     user.GetAccount().GetIdValue(),
		Property: request.GetProperty(),
		Statuses: []invoice.InvoiceStatus{
			invoice.InvoiceStatus_INVOICE_FULFILLED,
		},
	})
	if err != nil {
		return products, nil
	}

	return &cardpayment.GetPropertyResponse{
		Payments: RenderCardpaymentReceipts(resp.GetInvoices()),
	}, nil
}

func CardPaymentMutationCancelResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &cardpayment.CancelReceiptRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.UserId = user.GetAccount().GetIdValue()
	return connections.CardPaymentService.CancelReceipt(params.Context, request)
}

func CardPaymentMutationCheckResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &cardpayment.CheckReceiptRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.UserId = user.GetAccount().GetIdValue()
	return connections.CardPaymentService.CheckReceipt(params.Context, request)
}

func CardPaymentMutationFormResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &CardpaymentFormArguments{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.UserId = user.GetAccount().GetIdValue()
	request.UserEmail = user.GetAccount().GetContactValue().GetEmailPrimaryValue()

	var services []*extras.ServiceSnapshot

	for _, id := range request.Items {
		item, err := connections.ExtrasService.Get(params.Context, &extras.GetRequest{
			Id:         id,
			PropertyId: request.GetProperty(),
		})
		if err != nil {
			return nil, err
		}

		services = append(services, extras.SnapshotService(item.GetExtraService()))
	}

	resp, err := connections.PropertyService.Get(params.Context, &property.GetRequest{
		Id:     request.GetProperty(),
		User:   user.GetAccount().GetIdValue(),
		Verify: false,
	})
	if err != nil {
		return nil, err
	}

	inv, err := connections.InvoiceService.InvoiceCreate(params.Context, &invoice.InvoiceCreateRequest{
		CorrelationType:   "property",
		CorrelationId:     request.GetProperty(),
		CorrelationTarget: "listing-services",
		Property:          request.GetProperty(),
		Ivr:               resp.GetProperty().GetIvr().GetValue(),
		User:              user.GetAccount().GetIdValue(),
		Services:          services,
		NotifyCallback:    "",
	})
	if err != nil {
		return nil, err
	}

	typ := request.GetType()

	if typ == cardpayment.PaymentType_UNKNOWN_PAYMENT_TYPE {
		typ = cardpayment.PaymentType_CARD
	}

	pay, err := connections.InvoiceService.PaymentCreate(params.Context, &invoice.PaymentCreateRequest{
		InvoiceId: inv.GetInvoice().GetInvoiceId(),
		User:      user.GetAccount().GetIdValue(),
		System: &invoice.PaymentSystemDataRequest{
			Data: &invoice.PaymentSystemDataRequest_Stripe{
				Stripe: &invoice.PaymentSystemStripeRequest{
					Type: typ,
				},
			},
		},
	})
	if err != nil {
		return nil, err
	}

	return &cardpayment.FormReceiptResponse{
		Id:           pay.GetPayment().GetSystem().GetStripe().GetResponse().GetStripeId(),
		ClientSecret: pay.GetPayment().GetSystem().GetStripe().GetResponse().GetClientSecret(),
	}, nil
}

func CardPaymentMutationMarkResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &cardpayment.MarkReceiptPaidRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.UserId = user.GetAccount().GetIdValue()
	return connections.CardPaymentService.MarkReceiptPaid(params.Context, request)
}

func CardPaymentMutationMarkBankResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	req := &MarkReceiptPaidRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}
	request := &cardpayment.MarkReceiptPaidRequest{
		Id:     req.ID,
		UserId: user.GetAccount().GetIdValue(),
	}
	return connections.CardPaymentService.MarkReceiptPaid(params.Context, request)
}
