package resolvers

import (
	"github.com/graphql-go/graphql"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers/errs"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

type deleteRequest struct {
	ID string `json:"id,omitempty"`
}

func initDocument(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "DocumentQuery",
		Description: "Document query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Get document by id",
				Args:        util.BindArguments(document.GetRequest{}),
				Resolve:     DocumentQueryGetResolver,
				Type:        util.BindOutput(document.GetResponse{}, "Document with given id", nil),
			},
			"list": &graphql.Field{
				Description: "Get documents by filters",
				Args:        util.BindArguments(document.ListRequest{}),
				Resolve:     DocumentQueryListResolver,
				Type:        util.BindOutput(document.ListResponse{}, "Document list", nil),
			},
		},
	}))

	(*rootQueries)["document"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "DocumentMutation",
		Description: "Document mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Create document",
				Args:        util.BindArguments(document.CreateRequest{}),
				Resolve:     DocumentMutationCreateResolver,
				Type:        util.BindOutput(document.CreateResponse{}, "Document create response", nil),
			},
			"update": &graphql.Field{
				Description: "Update document",
				Args:        util.BindArguments(document.UpdateRequest{}),
				Resolve:     DocumentMutationUpdateResolver,
				Type:        util.BindOutput(document.UpdateResponse{}, "Document update response", nil),
			},
			"delete": &graphql.Field{
				Description: "Delete document",
				Args:        util.BindArguments(deleteRequest{}),
				Resolve:     DocumentMutationDeleteResolver,
				Type:        util.BindOutput(document.DeleteResponse{}, "Document delete response", nil),
			},
		},
	}))

	(*rootMutations)["document"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func DocumentQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &document.GetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.DocumentService.Get(params.Context, request)
}

func DocumentQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &document.ListRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	if request.GetProperty() != "" {
		user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
		if user == nil {
			return nil, errs.ErrNoUser
		}

		resp, err := connections.PropertyService.Get(params.Context, &property.GetRequest{
			Id:   request.GetProperty(),
			User: user.GetAccount().GetIdValue(),
		})

		if err != nil {
			return nil, err
		}

		types := propertyDocumentTypes(resp.GetProperty(), user.GetAccount().GetIdValue())
		if len(types) == 0 {
			return &document.ListResponse{}, nil
		}

		var actualTypes []document.DocumentType

		if len(request.GetTypes()) == 0 {
			actualTypes = types
		} else {
			for _, t := range types {
				for _, r := range request.GetTypes() {
					if r == t {
						actualTypes = append(actualTypes, t)
						break
					}
				}
			}
		}

		request.User = user.GetAccount().GetIdValue()
		request.Types = actualTypes
	}

	return connections.DocumentService.List(params.Context, request)
}

func DocumentMutationCreateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &document.CreateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	return connections.DocumentService.Create(params.Context, request)
}

func DocumentMutationUpdateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &document.UpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	return connections.DocumentService.Update(params.Context, request)
}

func DocumentMutationDeleteResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	req := &deleteRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	userID := user.GetAccount().GetIdValue()

	// Delete document from account
	reqAcc := &account.DeleteDocumentRequest{
		Id:        req.ID,
		User:      userID,
		AdminSite: false,
	}
	if resp, err := connections.AccountService.DeleteDocument(
		params.Context,
		reqAcc,
	); (resp != nil) && resp.GetError() {
		return nil, err
	}

	// Delete document from property
	reqProp := &property.DeleteDocumentRequest{
		Id:        req.ID,
		User:      userID,
		AdminSite: false,
	}
	if resp, err := connections.PropertyService.DeleteDocument(
		params.Context,
		reqProp,
	); resp != nil && resp.GetError() {
		return nil, err
	}

	// Delete document from offer
	reqOffer := &offer.DeleteDocumentRequest{
		Id:        req.ID,
		User:      userID,
		AdminSite: false,
	}
	if resp, err := connections.OfferService.DeleteDocument(
		params.Context,
		reqOffer,
	); resp != nil && resp.GetError() {
		return nil, err
	}

	// Delete document itself
	reqDoc := &document.DeleteRequest{
		Id:        req.ID,
		User:      userID,
		AdminSite: false,
	}
	return connections.DocumentService.Delete(params.Context, reqDoc)
}
