package resolvers

import (
	"github.com/graphql-go/graphql"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers/errs"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

func initInvitation(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "InvitationQuery",
		Description: "Invitation query root",
		Fields: graphql.Fields{
			"incoming": &graphql.Field{
				Description: "Get incoming invitations",
				Resolve:     InvitationIncomingQueryResolver,
				Type:        util.BindOutput(invitation.InvitationListResponse{}, "Invitations list", nil),
			},
			"outgoing": &graphql.Field{
				Description: "Get outgoing invitations",
				Resolve:     InvitationOutgoingQueryResolver,
				Type:        util.BindOutput(invitation.InvitationListResponse{}, "Invitations list", nil),
			},
		},
	}))

	(*rootQueries)["invitation"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "InvitationMutation",
		Description: "Invitation mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Create invitation",
				Args:        util.BindArguments(invitation.InvitationInfo{}),
				Resolve:     InvitationCreateMutationResolver,
				Type:        util.BindOutput(invitation.CreateInvitationResponse{}, "Create status", nil),
			},
			"accept": &graphql.Field{
				Description: "Accept invitation",
				Args:        util.BindArguments(invitation.InvitationActionRequest{}),
				Resolve:     InvitationAcceptMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Accept status", nil),
			},
			"reject": &graphql.Field{
				Description: "Reject invitation",
				Args:        util.BindArguments(invitation.InvitationActionRequest{}),
				Resolve:     InvitationRejectMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Reject status", nil),
			},
			"decline": &graphql.Field{
				Description: "Decline invitation by id, no auth",
				Args:        util.BindArguments(invitation.InvitationActionRequest{}),
				Resolve:     InvitationDeclineByLinkMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Decline status", nil),
			},
			"cancel": &graphql.Field{
				Description: "Cancel invitation by id for inviter",
				Args:        util.BindArguments(invitation.CancelInvitationByIdRequest{}),
				Resolve:     InvitationCancelByIDMutationResolver,
				Type:        util.BindOutput(invitation.InvitationActionResponse{}, "Cancel status", nil),
			},
			"referlandlord": &graphql.Field{
				Description: "Save refer landlord data to google sheet",
				Args:        util.BindArguments(invitation.ReferLandlordRequest{}),
				Resolve:     InvitationReferLandlordMutationResolver,
				Type:        util.BindOutput(invitation.ReferLandlordResponse{}, "Refer landlord status", nil),
			},
			"cancelall": &graphql.Field{
				Description: "Cancel all property invitations by property id and invitation type",
				Args:        util.BindArguments(invitation.CancelInvitationsByPropertyRequest{}),
				Resolve:     InvitationCancelAllMutationResolver,
				Type: util.BindOutput(
					invitation.CancelInvitationsByPropertyResponse{},
					"Cancel all invites response",
					nil,
				),
			},
		},
	}))

	(*rootMutations)["invitation"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func InvitationIncomingQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &invitation.InvitationListIncomingRequest{
		Email: user.GetAccount().GetContactValue().GetEmailPrimaryValue(),
	}
	return connections.InvitationService.InvitationListIncoming(params.Context, request)
}

func InvitationOutgoingQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &invitation.InvitationListOutgoingRequest{
		User: user.GetAccount().GetIdValue(),
	}
	return connections.InvitationService.InvitationListOutgoing(params.Context, request)
}

func InvitationCreateMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &invitation.InvitationInfo{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	prop, err := connections.PropertyService.Get(params.Context, &property.GetRequest{
		Id: request.GetPropertyId(),
	})
	if err != nil {
		return nil, err
	}

	inviter, err := connections.AccountService.Get(params.Context, &account.GetRequest{
		Id: user.GetAccount().GetIdValue(),
	})
	if err != nil {
		return nil, err
	}

	email := request.GetInviteeEmail()
	phone := request.GetInviteePhone()
	firstName := request.GetInviteeFirstName()
	lastName := request.GetInviteeLastName()
	message := request.GetMessage()

	if message == "" {
		message = request.Body
	}

	resp, err := connections.PropertyService.Invite(params.Context, &property.InviteRequest{
		Property:         prop.GetProperty().GetId().GetValue(),
		Type:             request.GetType().String(),
		InviterId:        inviter.GetAccount().GetIdValue(),
		InviteeId:        request.GetInviteeId(),
		InviteeEmail:     email,
		InviteePhone:     phone,
		InviteeFirstName: firstName,
		InviteeLastName:  lastName,
		SiteUrl:          connections.SiteURL,
		Message:          message,
	})
	if err != nil {
		return nil, err
	}

	return &invitation.CreateInvitationResponse{
		InvitationId: resp.GetId(),
		Type:         request.GetType(),
	}, nil
}

func InvitationAcceptMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &invitation.InvitationActionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Email = user.GetAccount().GetContactValue().GetEmailPrimaryValue()
	request.User = user.GetAccount().GetIdValue()
	return connections.InvitationService.AcceptInvitation(params.Context, request)
}

func InvitationRejectMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &invitation.InvitationActionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.Email = user.GetAccount().GetContactValue().GetEmailPrimaryValue()
	request.User = user.GetAccount().GetIdValue()
	return connections.InvitationService.DeclineInvitation(params.Context, request)
}

func InvitationDeclineByLinkMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &invitation.InvitationActionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvitationService.DeclineInvitationById(params.Context, request)
}

func InvitationCancelByIDMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &invitation.CancelInvitationByIdRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	request.Admin = HasRole(user, AdminRole)
	return connections.InvitationService.CancelInvitationById(params.Context, request)
}

func InvitationReferLandlordMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &invitation.ReferLandlordRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.InvitationService.ReferLandlord(params.Context, request)
}

func InvitationCancelAllMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}
	request := &invitation.CancelInvitationsByPropertyRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	request.User = user.GetAccount().GetIdValue()
	return connections.InvitationService.CancelInvitationsByProperty(params.Context, request)
}
