package resolvers

import (
	"github.com/graphql-go/graphql"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers/errs"
	maintenance "gitlab.mashroom.com/mashroom/backend/maintenance/api"
)

func initMaintenance(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "MaintenanceQuery",
		Description: "Maintenance query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Retrieves specified maintenance",
				Args:        util.BindArguments(maintenance.GetRequest{}),
				Resolve:     MaintenanceQueryGetResolver,
				Type: util.BindOutputName(
					maintenance.GetResponse{},
					"Maintenance information",
					util.EnrichmentMap{
						"success": &util.Enrichment{
							DeprecationReason: "Do not use",
							Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (interface{}, error) {
								return true, nil
							},
						},
					},
					"maintenance_api_GetResponse",
				),
			},
			"list": &graphql.Field{
				Description: "Retrieves owned maintenance",
				Args:        util.BindArguments(maintenance.ListRequest{}),
				Resolve:     MaintenanceQueryListResolver,
				Type: util.BindOutputName(
					maintenance.ListResponse{},
					"Maintenance list information",
					util.EnrichmentMap{
						"success": &util.Enrichment{
							DeprecationReason: "Do not use",
							Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (interface{}, error) {
								return true, nil
							},
						},
					},
					"maintenance_api_ListResponse",
				),
			},
			"incoming": &graphql.Field{
				DeprecationReason: "Use list with direction=INCOMING",
				Description:       "Retrieves owned maintenance",
				Args:              util.BindArguments(maintenance.ListRequest{}),
				Resolve:           MaintenanceQueryListIncomingResolver,
				Type:              util.BindOutput(maintenance.ListResponse{}, "Maintenance list information", nil),
			},
			"outgoing": &graphql.Field{
				DeprecationReason: "Use list with direction=OUTGOING",
				Description:       "Retrieves owned maintenance",
				Args:              util.BindArguments(maintenance.ListRequest{}),
				Resolve:           MaintenanceQueryListOutgoingResolver,
				Type:              util.BindOutput(maintenance.ListResponse{}, "Maintenance list information", nil),
			},
			"getsubcategories": &graphql.Field{
				DeprecationReason: "Use getjobdescriptions with query only",
				Description:       "Get subcategories of local heroes",
				Args:              util.BindArguments(maintenance.GetSubcategoryListRequest{}),
				Resolve:           MaintenanceQueryGetSubcategoriesResolver,
				Type: util.BindOutput(
					maintenance.GetSubcategoryListResponse{},
					"Subcategories list information",
					nil,
				),
			},
			"getjobdescriptions": &graphql.Field{
				Description: "Get job descriptions of local heroes for title",
				Args:        util.BindArguments(maintenance.GetJobDescriptionListRequest{}),
				Resolve:     MaintenanceQueryGetJobDescriptionsResolver,
				Type: util.BindOutputName(
					maintenance.GetJobDescriptionListResponse{},
					"Job descriptions list information",
					util.EnrichmentMap{
						"job_description": &util.Enrichment{
							DeprecationReason: "Use taxonomies",
							OutputNullable:    true,
						},
					},
					"maintenance_api_GetJobDescriptionListResponse",
				),
			},
		},
	}))

	(*rootQueries)["maintenance"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "MaintenanceMutation",
		Description: "Maintenance mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Creates new maintenance",
				Args:        util.BindArguments(maintenance.CreateRequest{}),
				Resolve:     MaintenanceMutationCreateResolver,
				Type: util.BindOutputName(maintenance.CreateResponse{}, "Create status", util.EnrichmentMap{
					"success": &util.Enrichment{
						DeprecationReason: "Do not use",
						Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (interface{}, error) {
							return true, nil
						},
					},
				}, "maintenance_api_CreateResponse"),
			},
			"update": &graphql.Field{
				Description: "Updates existing maintenance",
				Args:        util.BindArguments(maintenance.UpdateRequest{}),
				Resolve:     MaintenanceMutationUpdateResolver,
				Type: util.BindOutputName(maintenance.UpdateResponse{}, "Update status", util.EnrichmentMap{
					"success": &util.Enrichment{
						DeprecationReason: "Do not use",
						Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (interface{}, error) {
							return true, nil
						},
					},
				}, "maintenance_api_UpdateResponse"),
			},
			"remove": &graphql.Field{
				Description: "Removes existing maintenance task",
				Args:        util.BindArguments(maintenance.RemoveRequest{}),
				Resolve:     MaintenanceMutationRemoveResolver,
				Type:        graphql.Boolean,
			},
			"assignlh": &graphql.Field{
				Description: "Assign issue to local heroes",
				Args:        util.BindArguments(maintenance.AssignIssueLocalHeroesRequest{}),
				Resolve:     MaintenanceMutationAssignLocalHeroesResolver,
				Type: util.BindOutputName(
					maintenance.AssignIssueLocalHeroesResponse{},
					"Assign status", util.EnrichmentMap{
						"success": &util.Enrichment{
							DeprecationReason: "Do not use",
							Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (interface{}, error) {
								return true, nil
							},
						},
						"error": &util.Enrichment{
							DeprecationReason: "Do not use",
							Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (interface{}, error) {
								return maintenance.MaintenanceError_MAINTENANCE_UNKNOWN_ERROR, nil
							},
						},
					},
					"maintenance_api_AssignIssueLocalHeroesResponse",
				),
			},
			"assignpriv": &graphql.Field{
				Description: "Assign issue to private contractor",
				Args:        util.BindArguments(maintenance.AssignIssueOwnContractorRequest{}),
				Resolve:     MaintenanceMutationAssignPrivateContractorResolver,
				Type:        util.BindOutput(maintenance.AssignIssueOwnContractorResponse{}, "Assign status", nil),
			},
		},
	}))

	(*rootMutations)["maintenance"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func MaintenanceQueryGetJobDescriptionsResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &maintenance.GetJobDescriptionListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.MaintenanceService.GetJobDescriptionList(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceQueryGetSubcategoriesResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &maintenance.GetSubcategoryListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.MaintenanceService.GetSubcategoryList(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	request := &maintenance.GetRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	if user != nil {
		request.User = user.GetAccount().GetIdValue()
	}

	resp, err := connections.MaintenanceService.Get(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.ListRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.List(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceQueryListIncomingResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.ListIncomingRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.ListIncoming(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceQueryListOutgoingResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.ListOutgoingRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.ListOutgoing(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceMutationCreateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.CreateRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.Create(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceMutationUpdateResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.UpdateRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.Update(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceMutationRemoveResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return false, errs.ErrNoUser
	}

	request := &maintenance.RemoveRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return false, err
	}

	request.User = user.GetAccount().GetIdValue()

	_, err := connections.MaintenanceService.Remove(params.Context, request)
	if err != nil {
		return false, err
	}

	return true, nil
}

func MaintenanceMutationAssignLocalHeroesResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.AssignIssueLocalHeroesRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.AssignIssueLocalHeroes(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func MaintenanceMutationAssignPrivateContractorResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &maintenance.AssignIssueOwnContractorRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	resp, err := connections.MaintenanceService.AssignIssueOwnContractor(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
