package resolvers

import (
	"github.com/graphql-go/graphql"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	mortgage "gitlab.mashroom.com/mashroom/backend/mortgage/api"
)

func initMortgage(rootQueries, rootMutations *graphql.Fields) {
	queryLeads := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "MortgageLeadsQuery",
		Description: "Mortgage leads query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Get lead",
				Args:        util.BindArguments(mortgage.LeadGetRequest{}),
				Resolve:     MortgageLeadsGetQueryResolver,
				Type:        util.BindOutput(mortgage.LeadGetResponse{}, "Lead details", nil),
			},
			"list": &graphql.Field{
				Description: "List queryLeads",
				Args:        util.BindArguments(mortgage.LeadListRequest{}),
				Resolve:     MortgageLeadsListQueryResolver,
				Type:        util.BindOutput(mortgage.LeadListResponse{}, "Leads list", nil),
			},
			"getdeal": &graphql.Field{
				Description: "Get lead deal",
				Args:        util.BindArguments(mortgage.LeadDealGetRequest{}),
				Resolve:     MortgageLeadDealGetQueryResolver,
				Type:        util.BindOutput(mortgage.LeadDealGetResponse{}, "Lead deal details", nil),
			},
			"getdealnote": &graphql.Field{
				Description: "Get lead deal note",
				Args:        util.BindArguments(mortgage.LeadDealNoteGetRequest{}),
				Resolve:     MortgageLeadDealNoteGetQueryResolver,
				Type:        util.BindOutput(mortgage.LeadDealNoteGetResponse{}, "Lead deal note details", nil),
			},
		},
	}))

	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "MortgageQuery",
		Description: "Mortgage query root",
		Fields: graphql.Fields{
			"lenders": &graphql.Field{
				Description: "Get lenders list",
				Resolve:     MortgageLendersQueryResolver,
				Type: graphql.NewList(
					graphql.NewNonNull(util.BindOutput(mortgage.Lender{}, "lender details", nil)),
				),
			},
			"leads": &graphql.Field{
				Type: queryLeads,
				Resolve: func(p graphql.ResolveParams) (interface{}, error) {
					return queryLeads, nil
				},
			},
		},
	}))

	(*rootQueries)["mortgage"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "MortgageMutation",
		Description: "Mortgage mutation root",
		Fields: graphql.Fields{
			"source": &graphql.Field{
				Description: "Run mortgage sourcing",
				Resolve:     MortgageSourceMutationResolver,
				Args:        util.BindArguments(mortgage.SourceCriteria{}),
				Type:        util.BindOutput(mortgage.RunSourceResponse{}, "Run sourcing criteria", nil),
			},
			"book": &graphql.Field{
				Description: "Book a call",
				Resolve:     MortgageBookMutationResolver,
				Args:        util.BindArguments(mortgage.BookCallRequest{}),
				Type:        util.BindOutput(mortgage.BookCallResponse{}, "Book a call response", nil),
			},
		},
	}))

	(*rootMutations)["mortgage"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func MortgageLendersQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	resp, err := connections.MortgageService.LenderList(params.Context, &mortgage.LenderListRequest{})
	if err != nil {
		return nil, err
	}

	return resp.GetLenders(), nil
}

func MortgageLeadsGetQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user, ok := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)

	req := &mortgage.LeadGetRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	if ok && user != nil {
		req.UserId = user.GetAccount().GetIdValue()
	}

	return connections.MortgageService.LeadGet(params.Context, req)
}

func MortgageLeadsListQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user, ok := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)

	req := &mortgage.LeadListRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	if ok && user != nil {
		req.UserId = user.GetAccount().GetIdValue()
	}

	return connections.MortgageService.LeadList(params.Context, req)
}

func MortgageLeadDealGetQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	req := &mortgage.LeadDealGetRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	return connections.MortgageService.LeadDealGet(params.Context, req)
}

func MortgageLeadDealNoteGetQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	req := &mortgage.LeadDealNoteGetRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	return connections.MortgageService.LeadDealNoteGet(params.Context, req)
}

func MortgageSourceMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	criteria := &mortgage.SourceCriteria{}
	if err := util.UnmarshalArguments(params.Args, criteria); err != nil {
		return nil, err
	}

	return connections.MortgageService.RunSource(params.Context, &mortgage.RunSourceRequest{
		Criteria: criteria,
	})
}

func MortgageBookMutationResolver(params graphql.ResolveParams) (interface{}, error) {
	user, ok := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)

	req := &mortgage.BookCallRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	if ok && user != nil {
		req.UserId = user.GetAccount().GetIdValue()
	}

	return connections.MortgageService.BookCall(params.Context, req)
}
