package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	signature "gitlab.mashroom.com/mashroom/backend/signature/api"
)

func initSignature(rootQueries, rootMutations *graphql.Fields) {
	sigresp := util.BindOutput(signature.SignatureResponse{}, "signature response", nil)

	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "SignatureQuery",
		Description: "Signature query root",
		Fields: graphql.Fields{
			"info": &graphql.Field{
				Description: "Get signature info by signature request id",
				Args:        util.BindArguments(signature.GetInfoRequest{}),
				Resolve:     SignatureQueryInfoResolver,
				Type:        sigresp,
			},
			"pdf": &graphql.Field{
				Description: "Get pdf media object id by signature request id",
				Args:        util.BindArguments(signature.GetPDFRequest{}),
				Resolve:     SignatureQueryPdfResolver,
				Type:        util.BindOutput(signature.GetPDFResponse{}, "pdf media id", nil),
			},
		},
	}))

	(*rootQueries)["signature"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "SignatureMutation",
		Description: "Signature mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Create signature request",
				Args:        util.BindArguments(signature.CreateRequest{}),
				Resolve:     SignatureMutationCreateResolver,
				Type:        sigresp,
			},
			"cancel": &graphql.Field{
				Description: "Cancel signature request",
				Args:        util.BindArguments(signature.CancelRequest{}),
				Resolve:     SignatureMutationCancelResolver,
				Type:        util.BindOutput(signature.CancelResponse{}, "cancel status", nil),
			},
		},
	}))

	(*rootMutations)["signature"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func SignatureQueryInfoResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &signature.GetInfoRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.SignatureService.GetInfo(params.Context, request)
}

func SignatureQueryPdfResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &signature.GetPDFRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.SignatureService.GetPDF(params.Context, request)
}

func SignatureMutationCreateResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &signature.CreateRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.SignatureService.Create(params.Context, request)
}

func SignatureMutationCancelResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &signature.CancelRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.SignatureService.Cancel(params.Context, request)
}
