package rest

import (
	"context"
	"fmt"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"google.golang.org/protobuf/types/known/wrapperspb"

	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
)

func queryOrDefaultInt32Wrap(values url.Values, key string, def *wrapperspb.Int32Value) *wrapperspb.Int32Value {
	vs := values[key]
	if len(vs) == 0 {
		return def
	}

	v, err := strconv.ParseInt(vs[0], 10, 32)
	if err != nil {
		return def
	}

	return wrapperspb.Int32(int32(v))
}

type propertyViewingParam struct {
	ViewID     string
	PropertyID string
	Action     string
	Token      string
	Comment    *wrapperspb.StringValue
	Rating     *wrapperspb.Int32Value
	Values     url.Values
}

func propertyViewingParamParse(request *http.Request, prefix string) (param *propertyViewingParam, err error) {
	next := strings.TrimPrefix(request.URL.Path, prefix)
	parts := strings.Split(next, "/")

	param = &propertyViewingParam{}

	switch len(parts) {
	case 0:
		return nil, fmt.Errorf("invalid action")
	case 1:
		param.Action = parts[0]
	default:
		param.ViewID = parts[0]
		param.Action = parts[1]
	}

	param.Values = request.URL.Query()

	param.Token = param.Values.Get("token")

	if len(param.Token) == 0 {
		return nil, fmt.Errorf("invalid token")
	}

	if vs, ok := param.Values["comment"]; ok {
		if len(vs) > 0 {
			param.Comment = wrapperspb.String(vs[0])
		}
	}

	if vs, ok := param.Values["rating"]; ok {
		var v int64

		if len(vs) > 0 {
			v, err = strconv.ParseInt(vs[0], 10, 32)
			if err != nil {
				return nil, err
			}

			param.Rating = wrapperspb.Int32(int32(v))
		}
	}

	return
}

func propertyViewingHandleAction(ctx context.Context, param *propertyViewingParam) error {
	switch param.Action {
	case "accept":
		resp, err := connections.PropertyService.AcceptView(ctx, &property.AcceptViewRequest{
			Id:      param.ViewID,
			Comment: param.Comment,
			Token:   param.Token,
		})
		if err != nil {
			return err
		}

		param.ViewID, param.PropertyID = resp.GetRequest().GetId(), resp.GetRequest().GetProperty()
	case "reject":
		resp, err := connections.PropertyService.RejectView(ctx, &property.RejectViewRequest{
			Id:      param.ViewID,
			Comment: param.Comment,
			Token:   param.Token,
		})
		if err != nil {
			return err
		}

		param.ViewID, param.PropertyID = resp.GetRequest().GetId(), resp.GetRequest().GetProperty()
	case "cancel":
		resp, err := connections.PropertyService.CancelView(ctx, &property.CancelViewRequest{
			Id:      param.ViewID,
			Comment: param.Comment,
			Token:   param.Token,
		})
		if err != nil {
			return err
		}

		param.ViewID, param.PropertyID = resp.GetRequest().GetId(), resp.GetRequest().GetProperty()
	case "complete":
		resp, err := connections.PropertyService.CompleteView(ctx, &property.CompleteViewRequest{
			Id:      param.ViewID,
			Comment: param.Comment,
			Token:   param.Token,
			Rating:  param.Rating,
		})
		if err != nil {
			return err
		}

		param.ViewID, param.PropertyID = resp.GetRequest().GetId(), resp.GetRequest().GetProperty()
	case "fail":
		resp, err := connections.PropertyService.FailView(ctx, &property.FailViewRequest{
			Id:      param.ViewID,
			Comment: param.Comment,
			Token:   param.Token,
		})
		if err != nil {
			return err
		}

		param.ViewID, param.PropertyID = resp.GetRequest().GetId(), resp.GetRequest().GetProperty()
	case "reschedule":
		view, err := connections.PropertyService.GetView(ctx, &property.GetViewRequest{
			Id:    param.ViewID,
			Token: param.Token,
		})
		if err != nil {
			return err
		}

		param.ViewID, param.PropertyID = view.GetRequest().GetId(), view.GetRequest().GetProperty()

		req := view.GetRequest()
		if req.Time == nil {
			req.Time = &property.ViewRequestTime{}
		}

		req.Time.Year = queryOrDefaultInt32Wrap(param.Values, "year", req.GetTime().GetYear())
		req.Time.Month = queryOrDefaultInt32Wrap(param.Values, "month", req.GetTime().GetMonth())
		req.Time.Day = queryOrDefaultInt32Wrap(param.Values, "day", req.GetTime().GetDay())
		req.Time.TimeStartsec = queryOrDefaultInt32Wrap(param.Values, "startsec", req.GetTime().GetTimeStartsec())
		req.Time.TimeEndsec = queryOrDefaultInt32Wrap(param.Values, "endsec", req.GetTime().GetTimeEndsec())

		_, err = connections.PropertyService.RequestView(ctx, &property.RequestViewRequest{
			Request: req,
			Comment: param.Comment,
			Token:   param.Token,
		})
		if err != nil {
			return err
		}
	default:
		return fmt.Errorf("unknown action")
	}

	return nil
}

func PropertyViewing(prefix, success, failure string) http.HandlerFunc {
	return func(writer http.ResponseWriter, request *http.Request) {
		param, err := propertyViewingParamParse(request, prefix)
		if err != nil {
			http.Redirect(writer, request, failure+"?error="+url.QueryEscape(err.Error()), http.StatusFound)
			return
		}

		err = propertyViewingHandleAction(request.Context(), param)
		if err != nil {
			http.Redirect(writer, request, failure+"?error="+url.QueryEscape(err.Error()), http.StatusFound)
			return
		}

		target := strings.NewReplacer("$view", param.ViewID, "$property", param.PropertyID).Replace(success)

		http.Redirect(writer, request, target, http.StatusFound)
	}
}
