package rest

import (
	"context"
	"crypto/sha256"
	"errors"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"strings"

	cardpayment "gitlab.mashroom.com/mashroom/backend/cardpayment/api"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	signature "gitlab.mashroom.com/mashroom/backend/signature/api"
)

func StripeHandler(req *http.Request) (int, string, error) {
	err := req.ParseMultipartForm(MultipartMemoryBuffer)
	if err != nil {
		return Err500Result(err)
	}

	jsonList, ok := req.MultipartForm.Value["json"]
	if !ok {
		return Err500Result(err)
	}
	jsonStr := strings.Join(jsonList, "")

	log.Info(jsonStr)

	data := []byte(jsonStr)

	actual := sha256.New().Sum(data)

	stream, err := connections.CardPaymentService.HandleCallback(context.Background())
	if err != nil {
		return Err500Result(err)
	}

	err = stream.Send(&cardpayment.HandleCallbackRequest{
		Init: &cardpayment.HandleCallbackInit{
			Sha256: actual,
		},
	})
	if err != nil {
		return Err500Result(err)
	}

	for len(data) > 0 {
		var trans []byte
		if len(data) > signature.ChunkSize {
			trans = data[:signature.ChunkSize]
			data = data[signature.ChunkSize:]
		} else {
			trans = data
			data = nil
		}
		err = stream.Send(&cardpayment.HandleCallbackRequest{
			Chunk: &cardpayment.DataChunk{
				Data: trans,
			},
		})

		if err != nil {
			return Err500Result(err)
		}
	}

	err = stream.CloseSend()
	if err != nil && !errors.Is(err, io.EOF) {
		return Err500Result(err)
	}

	return http.StatusOK, "", nil
}

func StripeWebhooksHandler(w http.ResponseWriter, req *http.Request) {
	const MaxBodyBytes = int64(65536)
	req.Body = http.MaxBytesReader(w, req.Body, MaxBodyBytes)
	body, err := ioutil.ReadAll(req.Body)
	if err != nil {
		log.Errorf("StripeWebhooksHandler error reading request body: %v\n", err)
		w.WriteHeader(http.StatusServiceUnavailable)
		return
	}

	resp, err := connections.CardPaymentService.HandleWebhook(req.Context(), &cardpayment.HandleWebhookRequest{
		StripeSignature: req.Header.Get("Stripe-Signature"),
		Data:            body,
	})

	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	switch resp.Status {
	case cardpayment.HttpStatusCode_OK:
		w.WriteHeader(http.StatusOK)
		return
	default:
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
}

// StripeSuccessHandler - Successfully completed checkout session
func StripeSuccessHandler(w http.ResponseWriter, req *http.Request) {
	if req.URL == nil {
		log.Errorln("StripeSuccessHandler empty URL")
		// TO DO - SET CORRECT LOCATION REDIRECT
		w.Header().Set("Location", dashboardURL)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}

	sessionID := req.URL.RawQuery
	if sessionID == "" {
		log.Errorln("StripeSuccessHandler session ID empty")
		// TO DO - SET CORRECT LOCATION REDIRECT
		w.Header().Set("Location", dashboardURL)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}

	strArr := strings.Split(sessionID, "=")
	if len(strArr) < 2 {
		log.Errorln("StripeSuccessHandler session ID empty")
		// TO DO - SET CORRECT LOCATION REDIRECT
		w.Header().Set("Location", dashboardURL)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}

	sessionID = strArr[1]
	resp, err := connections.CardPaymentService.HandleCheckout(req.Context(), &cardpayment.HandleCheckoutRequest{
		SessionId: sessionID,
		Success:   true,
	})

	backLink := fmt.Sprintf(
		dashboardURL,
		connections.SiteURL,
	)

	if err != nil {
		log.Errorf("StripeCancelHandler error HandleCheckout: %v\n", err)
		w.Header().Set("Location", backLink)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}
	if !resp.Success {
		log.Errorf("StripeSuccessHandler not found session %s data\n", sessionID)
		w.Header().Set("Location", backLink)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}
	if resp.Property == "" {
		log.Errorf("StripeSuccessHandler not found property %s data\n", sessionID)
		w.Header().Set("Location", backLink)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}
	backLink = fmt.Sprintf(
		dashboardURL+"/%s/overview",
		connections.SiteURL,
		resp.Property,
	)

	w.Header().Set("Location", backLink)
	w.WriteHeader(http.StatusMovedPermanently)
}

// StripeCancelHandler - Interrupted checkout session
func StripeCancelHandler(w http.ResponseWriter, req *http.Request) {
	if req.URL == nil {
		log.Errorln("StripeCancelHandler empty URL")
		// TO DO - SET CORRECT LOCATION REDIRECT
		w.Header().Set("Location", dashboardURL)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}

	sessionID := req.URL.RawQuery
	if sessionID == "" {
		log.Errorln("StripeCancelHandler session ID empty")
		// TO DO - SET CORRECT LOCATION REDIRECT
		w.Header().Set("Location", dashboardURL)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}

	backLink := fmt.Sprintf(
		dashboardURL,
		connections.SiteURL,
	)

	strArr := strings.Split(sessionID, "=")
	sessionID = strArr[1]
	resp, err := connections.CardPaymentService.HandleCheckout(req.Context(), &cardpayment.HandleCheckoutRequest{
		SessionId: sessionID,
		Success:   true,
	})
	if err != nil {
		log.Errorf("StripeCancelHandler error HandleCheckout: %v\n", err)
		w.Header().Set("Location", backLink)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}
	if !resp.Success {
		log.Errorf("StripeSuccessHandler not found session %s data\n", sessionID)
		w.Header().Set("Location", backLink)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}
	if resp.Property == "" {
		log.Errorf("StripeSuccessHandler not found property %s data\n", sessionID)
		w.Header().Set("Location", backLink)
		w.WriteHeader(http.StatusMovedPermanently)
		return
	}
	backLink = fmt.Sprintf(
		dashboardURL+"/%s/overview",
		connections.SiteURL,
		resp.Property,
	)

	w.Header().Set("Location", backLink)
	w.WriteHeader(http.StatusMovedPermanently)
}
